(function (Drupal, once, drupalSettings) {
  'use strict';

  function toggleMenu(toggle, menu) {
    // Set whether the toggle is expanded.
    if (toggle.getAttribute('aria-expanded') === 'false') {
      toggle.setAttribute('aria-expanded', 'true');
    } else {
      toggle.setAttribute('aria-expanded', 'false');
    }
    // Toggle the menu open or closed.
    menu.classList.toggle('hide');
    menu.style.display = menu.style.display == 'none' ? '' : 'none';
  }

  function hideMenu(toggle, menu) {
    // Set the toggle to not expanded.
    toggle.setAttribute('aria-expanded', 'false');
    // Hide the menu
    menu.classList.add('hide');
    menu.style.display = 'none';
  }

  function showMenu(toggle, menu) {
    // Set the toggle to expanded.
    toggle.setAttribute('aria-expanded', 'true');
    // Show the menu
    menu.classList.remove('hide');
    menu.style.display = '';
  }

  function configureToggleButton(toggle) {
    const menuID = toggle.getAttribute('aria-controls');
    const menu = document.getElementById(menuID);
    // Hide button if there's no corresponding menu.
    if (!menu) {
      toggle.style.display = 'none';
    } else {
      // On toggle button click, toggle the menu.
      toggle.addEventListener('click', () => toggleMenu(toggle, menu));

      // Close menu after tabbing past it.
      menu.addEventListener('focusout', (event) => {
        if (!menu.contains(event.relatedTarget)) hideMenu(toggle, menu);
      });
      toggle.addEventListener('blur', (event) => {
        if (!menu.contains(event.relatedTarget)) hideMenu(toggle, menu);
      });
    }
  }

  // Keyboard Navigation.
  function disclosureMenuNav(id, menuSettings, context) {
    once('js-toggle-init', `#${id} button.menu-toggle`, context).forEach(
      (toggle) => configureToggleButton(toggle)
    );
  }

  // Hover Navigation.
  function disclosureMenuHoverNav(id, menuSettings, context) {
    once(
      'js-hover-init',
      `#${id} li.menu-item > a[data-submenu-id]`,
      context
    ).forEach((link) => {
      const submenuID = link.getAttribute('data-submenu-id');
      const submenu = document.getElementById(submenuID);
      const toggle = document.querySelector(
        `button[aria-controls="${submenuID}"]`
      );
      if (!submenu || !toggle) return;

      // Set timeout to hide submenu.
      const startHideIntent = () => {
        const hideIntent = setTimeout(
          () => hideMenu(toggle, submenu),
          menuSettings['hoverHideDelay']
        );
        submenu.setAttribute('data-hide-intent', hideIntent);
      };

      // Remove timeout to hide submenu, if it exists.
      const endHideIntent = () => {
        const hideIntent = submenu.getAttribute('data-hide-intent');
        if (hideIntent) {
          clearTimeout(hideIntent);
          submenu.removeAttribute('data-hide-intent');
        }
      };

      // Set timeout to show submenu.
      const startShowIntent = () => {
        const showIntent = setTimeout(
          () => showMenu(toggle, submenu),
          menuSettings['hoverShowDelay']
        );
        submenu.setAttribute('data-show-intent', showIntent);
      };

      // Remove timeout to show submenu, if it exists.
      const endShowIntent = () => {
        const showIntent = submenu.getAttribute('data-show-intent');
        if (showIntent) {
          clearTimeout(showIntent);
          submenu.removeAttribute('data-show-intent');
        }
      };

      // When hover over parent link, show submenu.
      link.addEventListener('pointerover', () => {
        endHideIntent();
        startShowIntent();
      });

      // When not hovering over parent link, hide submenu.
      link.addEventListener('pointerout', () => {
        endShowIntent();
        startHideIntent();
      });

      // When hovering over submenu, keep showing submenu.
      submenu.addEventListener('pointerover', endHideIntent);

      // When not hovering over submenu, hide submenu.
      submenu.addEventListener('pointerout', startHideIntent);
    });
  }

  Drupal.behaviors.disclosure_menu = {
    attach: function (context, settings) {
      once(
        'js-menu-disclosure-init',
        'button.full-menu-toggle[aria-controls]',
        context
      ).forEach((menuToggle) => configureToggleButton(menuToggle));

      // Iterate through disclosure menus and apply navigation settings.
      for (const id in drupalSettings.disclosureMenu) {
        if (Object.hasOwnProperty.call(drupalSettings.disclosureMenu, id)) {
          const menuSettings = drupalSettings.disclosureMenu[id];
          disclosureMenuNav(id, menuSettings, context);
          if (menuSettings.hover) {
            disclosureMenuHoverNav(id, menuSettings, context);
          }
        }
      }
    },
    detach: function (context, settings) {},
  };
})(Drupal, once, drupalSettings);
